package sun.security.x509;

import java.security.cert.CertificateException;
import java.util.Iterator;
import java.util.Enumeration;
import java.util.Collections;
import sun.security.util.DerOutputStream;
import java.io.OutputStream;
import java.util.ArrayList;
import sun.security.util.DerValue;
import sun.security.util.ObjectIdentifier;
import java.io.IOException;
import java.util.List;

public class CRLDistributionPointsExtension extends Extension implements CertAttrSet<String>
{
    public static final String IDENT = "x509.info.extensions.CRLDistributionPoints";
    public static final String NAME = "CRLDistributionPoints";
    public static final String POINTS = "points";
    private List<DistributionPoint> distributionPoints;
    private String extensionName;
    
    public CRLDistributionPointsExtension(List<DistributionPoint> list) throws IOException {
        this(false, list);
    }
    
    public CRLDistributionPointsExtension(boolean b, List<DistributionPoint> list) throws IOException {
        this(PKIXExtensions.CRLDistributionPoints_Id, b, list, "CRLDistributionPoints");
    }
    
    protected CRLDistributionPointsExtension(ObjectIdentifier extensionId, boolean critical, List<DistributionPoint> distributionPoints, String extensionName) throws IOException {
        this.extensionId = extensionId;
        this.critical = critical;
        this.distributionPoints = distributionPoints;
        this.encodeThis();
        this.extensionName = extensionName;
    }
    
    public CRLDistributionPointsExtension(Boolean b, Object o) throws IOException {
        this(PKIXExtensions.CRLDistributionPoints_Id, b, o, "CRLDistributionPoints");
    }
    
    protected CRLDistributionPointsExtension(ObjectIdentifier extensionId, Boolean b, Object o, String s) throws IOException {
        this.extensionId = extensionId;
        this.critical = (boolean)b;
        if (!(o instanceof byte[])) {
            throw new IOException("Illegal argument type");
        }
        this.extensionValue = (byte[])(byte[])o;
        DerValue derValue = new DerValue(this.extensionValue);
        if (derValue.tag != 48) {
            throw new IOException("Invalid encoding for " + s + " extension.");
        }
        this.distributionPoints = new ArrayList();
        while (derValue.data.available() != 0) {
            this.distributionPoints.add(new DistributionPoint(derValue.data.getDerValue()));
        }
        this.extensionName = s;
        return;
    }
    
    @Override
    public String getName() {
        return this.extensionName;
    }
    
    @Override
    public void encode(OutputStream outputStream) throws IOException {
        this.encode(outputStream, PKIXExtensions.CRLDistributionPoints_Id, false);
    }
    
    protected void encode(OutputStream outputStream, ObjectIdentifier extensionId, boolean critical) throws IOException {
        DerOutputStream derOutputStream = new DerOutputStream();
        if (this.extensionValue == null) {
            this.extensionId = extensionId;
            this.critical = critical;
            this.encodeThis();
        }
        super.encode(derOutputStream);
        outputStream.write(derOutputStream.toByteArray());
    }
    
    @Override
    public void set(String str, Object o) throws IOException {
        if (str.equalsIgnoreCase("points")) {
            if (!(o instanceof List)) {
                throw new IOException("Attribute value should be of type List.");
            }
            this.distributionPoints = (List)o;
            this.encodeThis();
            return;
        }
        else {
            throw new IOException("Attribute name [" + str + "] not recognized by CertAttrSet:" + this.extensionName + ".");
        }
    }
    
    @Override
    public List<DistributionPoint> get(String str) throws IOException {
        if (str.equalsIgnoreCase("points")) {
            return this.distributionPoints;
        }
        throw new IOException("Attribute name [" + str + "] not recognized by CertAttrSet:" + this.extensionName + ".");
    }
    
    @Override
    public void delete(String str) throws IOException {
        if (str.equalsIgnoreCase("points")) {
            this.distributionPoints = Collections.emptyList();
            this.encodeThis();
            return;
        }
        throw new IOException("Attribute name [" + str + "] not recognized by CertAttrSet:" + this.extensionName + 46);
    }
    
    @Override
    public Enumeration<String> getElements() {
        AttributeNameEnumeration attributeNameEnumeration = new AttributeNameEnumeration();
        attributeNameEnumeration.addElement("points");
        return attributeNameEnumeration.elements();
    }
    
    private void encodeThis() throws IOException {
        if (this.distributionPoints.isEmpty()) {
            this.extensionValue = null;
        }
        else {
            DerOutputStream derOutputStream = new DerOutputStream();
            Iterator<DistributionPoint> iterator = this.distributionPoints.iterator();
            while (iterator.hasNext()) {
                ((DistributionPoint)iterator.next()).encode(derOutputStream);
            }
            DerOutputStream o = new DerOutputStream();
            o.write((byte)48, derOutputStream);
            this.extensionValue = o.toByteArray();
        }
    }
    
    @Override
    public String toString() {
        return super.toString() + this.extensionName + " [\n  " + this.distributionPoints + "]\n";
    }
}
